"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Messenger_1 = require("../Messenger");
const utils_1 = require("../utils");
const readline = __importStar(require("readline"));
const child_process_1 = require("child_process");
let proc;
function main(args, conf, ks) {
    return __awaiter(this, void 0, void 0, function* () {
        const promises = [];
        const messenger = new Messenger_1.Messenger(conf);
        messenger.on('close', () => {
            console.error(`The connection to KISS TNC at ${conf.kissPort} is now closed. Exiting.`);
            process.exit(1);
        });
        messenger.on('tnc-error', (err) => {
            console.error(`The connection to KISS TNC ${conf.kissPort} experienced the following error:`);
            console.error(err);
        });
        messenger.on('message', (message) => {
            let receive = false;
            const to = utils_1.callsignSSIDToStation(args.to);
            if (args.allowAll)
                receive = true;
            else if (args.allRecipients ||
                (message.to.callsign === to.callsign && message.to.ssid == to.ssid)) {
                if (message.verification === Messenger_1.Verification.Valid) {
                    receive = true;
                }
                else if (args.allowUnsigned && message.verification === Messenger_1.Verification.NotSigned) {
                    receive = true;
                }
                else if (args.allowUntrusted && message.verification === Messenger_1.Verification.KeyNotFound) {
                    receive = true;
                }
                else if (args.allowInvalid && message.verification === Messenger_1.Verification.Invalid) {
                    receive = true;
                }
            }
            if (receive) {
                promises.push(utils_1.timeout(args.delay).then(() => writeToProc(proc, message.message)));
            }
        });
        try {
            yield messenger.openTNC();
        }
        catch (err) {
            console.error(`Error opening a connection to the KISS TNC that should be listening at ${conf.kissPort}. Are you sure your TNC is running?`);
            console.error(`If you have Direwolf installed you can start it in another window with "direwolf -p -q d -t 0"`);
            return 1;
        }
        if (!utils_1.isCallsign(args.to) && !utils_1.isCallsignSSID(args.to)) {
            console.error(`--to must be a valid callsign, callsign-ssid pair (e.g. "CALL-1"), or chatroom.`);
            return 1;
        }
        yield new Promise((resolve, reject) => {
            const commandArgs = args.command.split(' ');
            const command = commandArgs.shift();
            proc = child_process_1.spawn(command, commandArgs);
            proc.stdout.on('data', (data) => {
                const text = data.toString('utf8');
                console.log(text);
                promises.push(messenger.send(args.to.toUpperCase(), text, true));
            });
            if (args.stderr) {
                proc.stderr.on('data', (data) => {
                    const text = data.toString('utf8');
                    console.error(text);
                    promises.push(messenger.send(args.to.toUpperCase(), text, true));
                });
            }
            proc.on('exit', (code) => {
                resolve(code);
            });
            proc.on('error', (err) => {
                if (err.code == 'ENOENT') {
                    console.error(`"${command}" does not exist.`);
                }
                reject(err);
            });
            const rl = readline.createInterface({
                input: process.stdin,
                terminal: false,
                crlfDelay: Infinity
            });
            rl.on('line', (line) => {
                proc.stdin.write(line);
                proc.stdin.write('\n');
            });
        });
        yield Promise.all(promises);
        return 0;
    });
}
exports.main = main;
// If we have spawned a child process send it a SIGTERM signal.
// If it doesn't die send it a SIGKILL in 7 seconds.
// Reject the promise if it still isn't dead 3 seconds after that.
function cleanup() {
    return new Promise((resolve, reject) => {
        if (proc) {
            proc.on('close', resolve);
            proc.on('exit', resolve);
            proc.on('error', reject);
            proc.kill('SIGTERM');
            // note that these timeouts can still run even after resolve and
            // reject, so be careful what's put in here.
            setTimeout(() => {
                if (proc.pid) {
                    proc.kill('SIGKILL');
                    let message = 'Sent SIGKILL signal after child process didn\'t';
                    message += ' exit from SIGTERM. There may be a zombie process';
                    message += ' left over as a result.';
                    console.error(message);
                }
            }, 7000);
            // give up and reject the promise
            setTimeout(reject, 10000);
        }
        else {
            resolve();
        }
    });
}
exports.cleanup = cleanup;
function writeToProc(proc, text) {
    if (proc) {
        proc.stdin.write(text);
        proc.stdin.write('\n');
    }
}
//# sourceMappingURL=exec.js.map