"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const elliptic_1 = require("elliptic");
const curve = 'p192';
const ec = new elliptic_1.ec(curve);
function genKeyWith(needle) {
    let genKey = () => {
        const key = ec.genKeyPair();
        return {
            public: key.getPublic('hex'),
            private: key.getPrivate('hex'),
            curve,
        };
    };
    let key = genKey();
    while (!key.public.includes(needle)) {
        key = genKey();
    }
    return key;
}
function main(args, conf, ks) {
    return __awaiter(this, void 0, void 0, function* () {
        let { prefix, suffix, needle, min } = args;
        let requirements = [];
        needle = needle || prefix || suffix;
        if (!/^[0-9a-fA-F]+$/.test(needle)) {
            console.error(`Cannot generate a vanity key using a needle with non-hexidecimal characters.`);
            process.exit(1);
        }
        else {
            console.log(`Generating vanity key for ${conf.callsign} with needle "${needle}"`);
        }
        let key = genKeyWith(needle);
        if (prefix)
            requirements.push((key) => key.public.slice(2, 2 + prefix.length) === prefix);
        if (suffix)
            requirements.push((key) => key.public.endsWith(suffix));
        if (min)
            requirements.push((key) => key.public.match(RegExp(needle, 'g')).length >= min);
        while (!requirements.every(requirement => requirement(key))) {
            key = genKeyWith(needle);
        }
        console.dir(key);
        return 0;
    });
}
exports.main = main;
//# sourceMappingURL=genvanity.js.map