"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Messenger_1 = require("../Messenger");
const utils_1 = require("../utils");
const readline = __importStar(require("readline"));
function main(args, conf, ks) {
    return __awaiter(this, void 0, void 0, function* () {
        const messenger = new Messenger_1.Messenger(conf);
        messenger.on('close', () => {
            console.error(`The connection to KISS TNC at ${conf.kissPort} is now closed. Exiting.`);
            process.exit(1);
        });
        messenger.on('tnc-error', (err) => {
            console.error(`The connection to KISS TNC ${conf.kissPort} experienced the following error:`);
            console.error(err);
        });
        try {
            yield messenger.openTNC();
        }
        catch (err) {
            console.error(`Error opening a connection to the KISS TNC that should be listening at ${conf.kissPort}. Are you sure your TNC is running?`);
            console.error(`If you have Direwolf installed you can start it in another window with "direwolf -p -q d -t 0"`);
            return 1;
        }
        if (!utils_1.isCallsign(args.to) && !utils_1.isCallsignSSID(args.to)) {
            console.error(`--to must be a valid callsign, callsign-ssid pair (e.g. "CALL-1"), or chatroom.`);
            return 1;
        }
        // only sign if the user's config has a signing key
        const sign = (typeof conf.signingKey === 'string' && args.dontSign !== true);
        if (args.message != null && args.message.length > 0) {
            yield messenger.send(args.to.toUpperCase(), args.message, sign);
        }
        else {
            yield new Promise((resolve, reject) => {
                const rl = readline.createInterface({
                    input: process.stdin,
                    terminal: false,
                    crlfDelay: Infinity
                });
                const promises = [];
                rl.on('line', (line) => {
                    promises.push(messenger.send(args.to.toUpperCase(), line, sign));
                });
                rl.on('close', () => {
                    Promise.all(promises).then(resolve).catch(reject);
                });
            });
        }
        return 0;
    });
}
exports.main = main;
//# sourceMappingURL=send.js.map