"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
function timeout(millis) {
    return new Promise((resolve) => {
        setTimeout(resolve, millis);
    });
}
exports.timeout = timeout;
// MD5 is weak as hell. Don't use this for anything cryptographic.
function md5(buffer) {
    return crypto_1.createHash('md5').update(buffer).digest('hex');
}
exports.md5 = md5;
function stationToCallsignSSID(station) {
    return typeof station.ssid === 'number' && station.ssid !== 0 ?
        `${station.callsign}-${station.ssid}` : station.callsign;
}
exports.stationToCallsignSSID = stationToCallsignSSID;
/**
 * @function callsignSSIDToStation
 * @param  {string} callsignSSID
 * @throws {TypeError} If callsignSSID is invalid
 * @returns Station
 */
function callsignSSIDToStation(callsignSSID) {
    if (isCallsignSSID(callsignSSID)) {
        const [callsign, ssidStr] = callsignSSID.split('-');
        const ssid = parseInt(ssidStr);
        return { callsign: callsign, ssid: ssid };
    }
    else if (isCallsign(callsignSSID)) {
        return { callsign: callsignSSID, ssid: 0 };
    }
    else {
        throw TypeError(`Invalid callsignSSID: ${callsignSSID}`);
    }
}
exports.callsignSSIDToStation = callsignSSIDToStation;
// mainly lifted from node-ax25 utils.testCallsign
// callsign must be max 6 characters alphanumeric (no "-")
function isCallsign(callsign) {
    if (typeof callsign !== 'string' || callsign.length > 6)
        return false;
    callsign = callsign.toUpperCase().replace(/\s*$/g, "");
    // AX.25 has a built-in hard limit of six characters, which means a
    // seven-character callsign cannot be used in an AX.25 network.
    if (callsign.length < 1 || callsign.length > 6)
        return false;
    for (let c = 0; c < callsign.length; c++) {
        let a = callsign[c].charCodeAt(0);
        if ((a >= 48 && a <= 57) || (a >= 65 && a <= 90))
            continue;
        return false;
    }
    return true;
}
exports.isCallsign = isCallsign;
function isCallsignSSID(callsignSSID) {
    if ((callsignSSID.match(/-/g) || []).length !== 1)
        return false;
    const [callsign, ssid] = callsignSSID.split('-');
    return isCallsign(callsign) && isSSID(parseInt(ssid));
}
exports.isCallsignSSID = isCallsignSSID;
function isSSID(ssid) {
    if (typeof ssid !== 'number')
        throw TypeError('ssid must be a number type');
    return ssid >= 0 && ssid <= 15;
}
exports.isSSID = isSSID;
function isBrokenPipeError(err) {
    return err.code == 'EPIPE' || err.code == 'EIO';
}
exports.isBrokenPipeError = isBrokenPipeError;
//# sourceMappingURL=utils.js.map